package cmdLogin

import (
    "syscall"
    "golang.org/x/term"
    "nexus-common/params"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

import (
    "errors"
    "encoding/json"
)

type Login struct {
    Name string
}

func (cmd *Login)GetName() string {
    return cmd.Name
}

func (cmd *Login)GetDesc() []string {
    return []string{
        "Log in and obtain an access token."}
}

func (cmd *Login)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.Name+" email")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
}

func (cmd *Login)Run(args []string) int {
    if len(args) != 1 {
        cmd.PrintUsage()
        return 1
    }

    email := args[0]

    u.PrintErr(email+"'s password: ")
    bytePwd, err := term.ReadPassword(int(syscall.Stdin))
    u.PrintlnErr("")
    if err != nil {
        u.PrintlnErr(err.Error())
        return 1
    }
    pwd := string(bytePwd)

    token, err := GetToken(email, pwd)
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return 1
    }

    u.Println(token)
    return 0
}

/*
OLD version where pwd was specified as an argument on the command line

func (cmd *Login)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.Name+" email password")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
}

func (cmd *Login)Run(args []string) int {
    if len(args) != 2 {
        cmd.PrintUsage()
        return 1
    }

    email := args[0]
    pwd := args[1]
    token, err := GetToken(email, pwd)
    if err != nil {
        u.PrintlnErr("Error: "+err.Error())
        return 1
    }

    u.Println(token)
    return 0
}
*/

// Returns the token if the authentication was successful or an error if it wasn't.
// curl --cacert ca-cert.pem -X POST https://localhost:8000/login -H 'Content-Type: application/json' -d '{"email": "johndoe@nexus.org", "pwd":"pipomolo"}'
// Returns a jwt token if authentication was successful:
// {"token":"<jwt token>"}
func GetToken(user, pwd string) (string, error) {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    loginArgs := &params.Login {user, pwd}
    resp, err := client.R().SetBody(loginArgs).Post(host+"/login")
    if err != nil {
        return "", err
    }

    if resp.IsSuccess() {
        type Response struct {
            Token string
        }
        var response Response
        err = json.Unmarshal(resp.Body(), &response)
        if err != nil {
            return "", err
        }
        return response.Token, nil
    } else {
        type Response struct {
            Message string
        }
        var response Response
        err = json.Unmarshal(resp.Body(), &response)
        if err != nil {
            return "", err
        }
        return "", errors.New(response.Message)
    }
}
